package sun.awt.windows;

import java.beans.PropertyChangeEvent;
import java.awt.GraphicsConfiguration;
import java.awt.KeyboardFocusManager;
import java.util.LinkedList;
import sun.awt.Win32GraphicsEnvironment;
import java.awt.Color;
import java.awt.peer.ComponentPeer;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.GraphicsDevice;
import sun.awt.AWTAccessor;
import sun.awt.Win32GraphicsConfig;
import java.awt.GraphicsEnvironment;
import java.awt.Dialog;
import java.awt.image.DataBufferInt;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.AWTEventMulticaster;
import java.awt.AWTEvent;
import java.awt.event.WindowEvent;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Font;
import sun.java2d.pipe.Region;
import sun.awt.AppContext;
import sun.awt.Win32GraphicsDevice;
import java.util.List;
import sun.awt.SunToolkit;
import java.awt.Window;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeListener;
import sun.util.logging.PlatformLogger;
import sun.awt.DisplayChangedListener;
import java.awt.peer.WindowPeer;

public class WWindowPeer extends WPanelPeer implements WindowPeer, DisplayChangedListener
{
    private static final PlatformLogger log;
    private static final PlatformLogger screenLog;
    private WWindowPeer modalBlocker;
    private boolean isOpaque;
    private TranslucentWindowPainter painter;
    private static final StringBuffer ACTIVE_WINDOWS_KEY;
    private static PropertyChangeListener activeWindowListener;
    private static final PropertyChangeListener guiDisposedListener;
    private WindowListener windowListener;
    private volatile Window.Type windowType;
    private volatile int sysX;
    private volatile int sysY;
    private volatile int sysW;
    private volatile int sysH;
    private float opacity;
    
    private static native void initIDs();
    
    @Override
    protected void disposeImpl() {
        AppContext targetToAppContext = SunToolkit.targetToAppContext(this.target);
        synchronized (targetToAppContext) {
            List<Object> list = (List)targetToAppContext.get(WWindowPeer.ACTIVE_WINDOWS_KEY);
            if (list != null) {
                list.remove(this);
            }
        }
        ((Win32GraphicsDevice)this.getGraphicsConfiguration().getDevice()).removeDisplayChangedListener(this);
        synchronized (this.getStateLock()) {
            TranslucentWindowPainter painter = this.painter;
            if (painter != null) {
                painter.flush();
            }
        }
        super.disposeImpl();
    }
    
    @Override
    public void toFront() {
        this.updateFocusableWindowState();
        this._toFront();
    }
    
    private native void _toFront();
    
    @Override
    public native void toBack();
    
    private native void setAlwaysOnTopNative(boolean p0);
    
    public void setAlwaysOnTop(boolean alwaysOnTopNative) {
        if ((alwaysOnTopNative && ((Window)this.target).isVisible()) || !alwaysOnTopNative) {
            this.setAlwaysOnTopNative(alwaysOnTopNative);
        }
    }
    
    @Override
    public void updateAlwaysOnTopState() {
        this.setAlwaysOnTop(((Window)this.target).isAlwaysOnTop());
    }
    
    @Override
    public void updateFocusableWindowState() {
        this.setFocusableWindow(((Window)this.target).isFocusableWindow());
    }
    
    native void setFocusableWindow(boolean p0);
    
    public void setTitle(String s) {
        if (s == null) {
            s = "";
        }
        this._setTitle(s);
    }
    
    private native void _setTitle(String p0);
    
    public void setResizable(boolean b) {
        this._setResizable(b);
    }
    
    private native void _setResizable(boolean p0);
    
    WWindowPeer(Window window) {
        super(window);
        this.modalBlocker = null;
        this.windowType = Window.Type.NORMAL;
        this.sysX = 0;
        this.sysY = 0;
        this.sysW = 0;
        this.sysH = 0;
        this.opacity = 1.0f;
    }
    
    @Override
    void initialize() {
        super.initialize();
        this.updateInsets(this.insets_);
        if (((Window)this.target).getFont() == null) {
            Font defaultFont = WWindowPeer.defaultFont;
            ((Window)this.target).setFont(defaultFont);
            this.setFont(defaultFont);
        }
        ((Win32GraphicsDevice)this.getGraphicsConfiguration().getDevice()).addDisplayChangedListener(this);
        initActiveWindowsTracking((Window)this.target);
        this.updateIconImages();
        Shape shape = ((Window)this.target).getShape();
        if (shape != null) {
            this.applyShape(Region.getInstance(shape, null));
        }
        float opacity = ((Window)this.target).getOpacity();
        if (opacity < 1.0f) {
            this.setOpacity(opacity);
        }
        synchronized (this.getStateLock()) {
            this.isOpaque = true;
            this.setOpaque(((Window)this.target).isOpaque());
        }
    }
    
    native void createAwtWindow(WComponentPeer p0);
    
    void preCreate(WComponentPeer wComponentPeer) {
        this.windowType = ((Window)this.target).getType();
    }
    
    @Override
    void create(WComponentPeer wComponentPeer) {
        this.preCreate(wComponentPeer);
        this.createAwtWindow(wComponentPeer);
    }
    
    @Override
    final WComponentPeer getNativeParent() {
        return (WComponentPeer)WToolkit.targetToPeer(((Window)this.target).getOwner());
    }
    
    protected void realShow() {
        super.show();
    }
    
    @Override
    public void show() {
        this.updateFocusableWindowState();
        int alwaysOnTop = ((Window)this.target).isAlwaysOnTop();
        this.updateGC();
        this.realShow();
        this.updateMinimumSize();
        if (((Window)this.target).isAlwaysOnTopSupported() && alwaysOnTop) {
            this.setAlwaysOnTop((boolean)alwaysOnTop);
        }
        synchronized (this.getStateLock()) {
            if (!this.isOpaque) {
                this.updateWindow(true);
            }
        }
        WComponentPeer nativeParent = this.getNativeParent();
        if (nativeParent != null && nativeParent.isLightweightFramePeer()) {
            Rectangle bounds = this.getBounds();
            this.handleExpose(0, 0, bounds.width, bounds.height);
        }
    }
    
    native void updateInsets(Insets p0);
    
    static native int getSysMinWidth();
    
    static native int getSysMinHeight();
    
    static native int getSysIconWidth();
    
    static native int getSysIconHeight();
    
    static native int getSysSmIconWidth();
    
    static native int getSysSmIconHeight();
    
    native void setIconImagesData(int[] p0, int p1, int p2, int[] p3, int p4, int p5);
    
    synchronized native void reshapeFrame(int p0, int p1, int p2, int p3);
    
    public boolean requestWindowFocus(CausedFocusEvent.Cause cause) {
        return this.focusAllowedFor() && this.requestWindowFocus(cause == CausedFocusEvent.Cause.MOUSE_EVENT);
    }
    
    private native boolean requestWindowFocus(boolean p0);
    
    public boolean focusAllowedFor() {
        Window window = (Window)this.target;
        return window.isVisible() && window.isEnabled() && window.isFocusableWindow() && !this.isModalBlocked();
    }
    
    @Override
    void hide() {
        WindowListener windowListener = this.windowListener;
        if (windowListener != null) {
            windowListener.windowClosing(new WindowEvent((Window)this.target, 201));
        }
        super.hide();
    }
    
    @Override
    void preprocessPostEvent(AWTEvent awtEvent) {
        if ((awtEvent instanceof WindowEvent)) {
            WindowListener windowListener = this.windowListener;
            if (windowListener != null) {
                switch (awtEvent.getID()) {
                    case 201: {
                        windowListener.windowClosing((WindowEvent)awtEvent);
                        break;
                    }
                    case 203: {
                        windowListener.windowIconified((WindowEvent)awtEvent);
                        break;
                    }
                }
            }
        }
    }
    
    synchronized void addWindowListener(WindowListener b) {
        this.windowListener = AWTEventMulticaster.add(this.windowListener, b);
    }
    
    synchronized void removeWindowListener(WindowListener oldl) {
        this.windowListener = AWTEventMulticaster.remove(this.windowListener, oldl);
    }
    
    @Override
    public void updateMinimumSize() {
        Dimension minimumSize = null;
        if (((Component)this.target).isMinimumSizeSet()) {
            minimumSize = ((Component)this.target).getMinimumSize();
        }
        if (minimumSize != null) {
            int sysMinWidth = getSysMinWidth();
            int sysMinHeight = getSysMinHeight();
            sysMinWidth = ((minimumSize.width >= sysMinWidth) ? minimumSize.width : sysMinWidth);
            sysMinHeight = ((minimumSize.height >= sysMinHeight) ? minimumSize.height : sysMinHeight);
            this.setMinSize(sysMinWidth, sysMinHeight);
        }
        else {
            this.setMinSize(0, 0);
        }
    }
    
    @Override
    public void updateIconImages() {
        List<Image> iconImages = ((Window)this.target).getIconImages();
        if (iconImages == null || iconImages.size() == 0) {
            this.setIconImagesData(null, 0, 0, null, 0, 0);
        }
        else {
            int sysIconWidth = getSysIconWidth();
            int sysIconHeight = getSysIconHeight();
            int sysSmIconWidth = getSysSmIconWidth();
            int sysSmIconHeight = getSysSmIconHeight();
            DataBufferInt scaledIconData = SunToolkit.getScaledIconData(iconImages, sysIconWidth, sysIconHeight);
            DataBufferInt scaledIconData2 = SunToolkit.getScaledIconData(iconImages, sysSmIconWidth, sysSmIconHeight);
            if (scaledIconData != null && scaledIconData2 != null) {
                this.setIconImagesData(scaledIconData.getData(), sysIconWidth, sysIconHeight, scaledIconData2.getData(), sysSmIconWidth, sysSmIconHeight);
            }
            else {
                this.setIconImagesData(null, 0, 0, null, 0, 0);
            }
        }
    }
    
    native void setMinSize(int p0, int p1);
    
    public boolean isModalBlocked() {
        return this.modalBlocker != null;
    }
    
    @Override
    public void setModalBlocked(Dialog dialog, boolean b) {
        synchronized (((Component)this.getTarget()).getTreeLock()) {
            WWindowPeer modalBlocker = (WWindowPeer)dialog.getPeer();
            if (b) {
                this.modalBlocker = modalBlocker;
                if ((modalBlocker instanceof WFileDialogPeer)) {
                    ((WFileDialogPeer)modalBlocker).blockWindow(this);
                }
                else if ((modalBlocker instanceof WPrintDialogPeer)) {
                    ((WPrintDialogPeer)modalBlocker).blockWindow(this);
                }
                else {
                    this.modalDisable(dialog, modalBlocker.getHWnd());
                }
            }
            else {
                this.modalBlocker = null;
                if ((modalBlocker instanceof WFileDialogPeer)) {
                    ((WFileDialogPeer)modalBlocker).unblockWindow(this);
                }
                else if ((modalBlocker instanceof WPrintDialogPeer)) {
                    ((WPrintDialogPeer)modalBlocker).unblockWindow(this);
                }
                else {
                    this.modalEnable(dialog);
                }
            }
        }
    }
    
    native void modalDisable(Dialog p0, long p1);
    
    native void modalEnable(Dialog p0);
    
    public static long[] getActiveWindowHandles(Component target) {
        AppContext targetToAppContext = SunToolkit.targetToAppContext(target);
        if (targetToAppContext == null) {
            return null;
        }
        synchronized (targetToAppContext) {
            List<Object> list = (List)targetToAppContext.get(WWindowPeer.ACTIVE_WINDOWS_KEY);
            if (list == null) {
                return null;
            }
            long[] array = new long[list.size()];
            for (int i = 0; i < list.size(); ++i) {
                array[i] = ((WWindowPeer)list.get(i)).getHWnd();
            }
            return array;
        }
    }
    
    void draggedToNewScreen() {
        this.displayChanged();
    }
    
    public void updateGC() {
        int screenImOn = this.getScreenImOn();
        if (WWindowPeer.screenLog.isLoggable(PlatformLogger.Level.FINER)) {
            WWindowPeer.log.finer("Screen number: " + screenImOn);
        }
        Win32GraphicsDevice win32GraphicsDevice = (Win32GraphicsDevice)this.winGraphicsConfig.getDevice();
        GraphicsDevice[] screenDevices = GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices();
        Win32GraphicsDevice win32GraphicsDevice2;
        if (screenImOn >= screenDevices.length) {
            win32GraphicsDevice2 = (Win32GraphicsDevice)GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
        }
        else {
            win32GraphicsDevice2 = (Win32GraphicsDevice)screenDevices[screenImOn];
        }
        this.winGraphicsConfig = (Win32GraphicsConfig)win32GraphicsDevice2.getDefaultConfiguration();
        if (WWindowPeer.screenLog.isLoggable(PlatformLogger.Level.FINE) && this.winGraphicsConfig == null) {
            WWindowPeer.screenLog.fine("Assertion (winGraphicsConfig != null) failed");
        }
        if (win32GraphicsDevice != win32GraphicsDevice2) {
            win32GraphicsDevice.removeDisplayChangedListener(this);
            win32GraphicsDevice2.addDisplayChangedListener(this);
        }
        AWTAccessor.getComponentAccessor().setGraphicsConfiguration((Component)this.target, this.winGraphicsConfig);
    }
    
    @Override
    public void displayChanged() {
        SunToolkit.executeOnEventHandlerThread(this.target, this::updateGC);
    }
    
    @Override
    public void paletteChanged() {
    }
    
    private native int getScreenImOn();
    
    public final native void setFullScreenExclusiveModeState(boolean p0);
    
    public void grab() {
        this.nativeGrab();
    }
    
    public void ungrab() {
        this.nativeUngrab();
    }
    
    private native void nativeGrab();
    
    private native void nativeUngrab();
    
    private final boolean hasWarningWindow() {
        return ((Window)this.target).getWarningString() != null;
    }
    
    boolean isTargetUndecorated() {
        return true;
    }
    
    @Override
    public native void repositionSecurityWarning();
    
    @Override
    public void setBounds(int sysX, int sysY, int sysW, int sysH, int n) {
        super.setBounds(this.sysX = sysX, this.sysY = sysY, this.sysW = sysW, this.sysH = sysH, n);
    }
    
    @Override
    public void print(Graphics graphics) {
        Shape shape = AWTAccessor.getWindowAccessor().getShape((Window)this.target);
        if (shape != null) {
            graphics.setClip(shape);
        }
        super.print(graphics);
    }
    
    private void replaceSurfaceDataRecursively(Component component) {
        if ((component instanceof Container)) {
            Component[] components = ((Container)component).getComponents();
            for (int length = components.length, i = 0; i < length; ++i) {
                this.replaceSurfaceDataRecursively(components[i]);
            }
        }
        ComponentPeer peer = component.getPeer();
        if ((peer instanceof WComponentPeer)) {
            ((WComponentPeer)peer).replaceSurfaceDataLater();
        }
    }
    
    public final Graphics getTranslucentGraphics() {
        synchronized (this.getStateLock()) {
            return this.isOpaque ? null : this.painter.getBackBuffer(false).getGraphics();
        }
    }
    
    @Override
    public void setBackground(Color background) {
        super.setBackground(background);
        synchronized (this.getStateLock()) {
            if (!this.isOpaque && ((Window)this.target).isVisible()) {
                this.updateWindow(true);
            }
        }
    }
    
    private native void setOpacity(int p0);
    
    @Override
    public void setOpacity(float opacity) {
        if (!((SunToolkit)((Window)this.target).getToolkit()).isWindowOpacitySupported()) {
            return;
        }
        if (opacity < 0.0f || opacity > 1.0f) {
            throw new IllegalArgumentException("The value of opacity should be in the range [0.0f .. 1.0f].");
        }
        if (((this.opacity == 1.0f && opacity < 1.0f) || (this.opacity < 1.0f && opacity == 1.0f)) && !Win32GraphicsEnvironment.isVistaOS()) {
            this.replaceSurfaceDataRecursively((Component)this.getTarget());
        }
        this.opacity = opacity;
        int opacity2 = (int)(opacity * 255.0f);
        if (opacity2 < 0) {
            opacity2 = 0;
        }
        if (opacity2 > 255) {
            opacity2 = 255;
        }
        this.setOpacity(opacity2);
        synchronized (this.getStateLock()) {
            if (!this.isOpaque && ((Window)this.target).isVisible()) {
                this.updateWindow(true);
            }
        }
        return;
    }
    
    private native void setOpaqueImpl(boolean p0);
    
    @Override
    public void setOpaque(boolean isOpaque) {
        synchronized (this.getStateLock()) {
            if (this.isOpaque == isOpaque) {
                return;
            }
        }
        Window window = (Window)this.getTarget();
        if (!isOpaque) {
            SunToolkit sunToolkit = (SunToolkit)window.getToolkit();
            if (!sunToolkit.isWindowTranslucencySupported() || !sunToolkit.isTranslucencyCapable(window.getGraphicsConfiguration())) {
                return;
            }
        }
        int vistaOS = Win32GraphicsEnvironment.isVistaOS();
        if (this.isOpaque != isOpaque && vistaOS == 0) {
            this.replaceSurfaceDataRecursively(window);
        }
        synchronized (this.getStateLock()) {
            this.setOpaqueImpl(this.isOpaque = isOpaque);
            if (isOpaque) {
                TranslucentWindowPainter painter = this.painter;
                if (painter != null) {
                    painter.flush();
                    this.painter = null;
                }
            }
            else {
                this.painter = TranslucentWindowPainter.createInstance(this);
            }
        }
        if (!(vistaOS == 0)) {
            Shape shape = window.getShape();
            if (shape != null) {
                window.setShape(shape);
            }
        }
        if (window.isVisible()) {
            this.updateWindow(true);
        }
    }
    
    native void updateWindowImpl(int[] p0, int p1, int p2);
    
    @Override
    public void updateWindow() {
        this.updateWindow(false);
    }
    
    private void updateWindow(boolean b) {
        Window window = (Window)this.target;
        synchronized (this.getStateLock()) {
            if (this.isOpaque || !window.isVisible() || window.getWidth() <= 0 || window.getHeight() <= 0) {
                return;
            }
            TranslucentWindowPainter painter = this.painter;
            if (painter != null) {
                painter.updateWindow(b);
            }
            else if (WWindowPeer.log.isLoggable(PlatformLogger.Level.FINER)) {
                WWindowPeer.log.finer("Translucent window painter is null in updateWindow");
            }
        }
    }
    
    private static void initActiveWindowsTracking(Window window) {
        AppContext appContext = AppContext.getAppContext();
        synchronized (appContext) {
            if ((List)appContext.get(WWindowPeer.ACTIVE_WINDOWS_KEY) == null) {
                appContext.put(WWindowPeer.ACTIVE_WINDOWS_KEY, new LinkedList());
                appContext.addPropertyChangeListener("guidisposed", WWindowPeer.guiDisposedListener);
                KeyboardFocusManager.getCurrentKeyboardFocusManager().addPropertyChangeListener("activeWindow", WWindowPeer.activeWindowListener);
            }
        }
    }
    
    static {
        log = PlatformLogger.getLogger("sun.awt.windows.WWindowPeer");
        screenLog = PlatformLogger.getLogger("sun.awt.windows.screen.WWindowPeer");
        ACTIVE_WINDOWS_KEY = new StringBuffer("active_windows_list");
        WWindowPeer.activeWindowListener = new WWindowPeer.ActiveWindowListener();
        guiDisposedListener = new WWindowPeer.GuiDisposedListener();
        initIDs();
    }
    
    private static class GuiDisposedListener implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            if (!(boolean)propertyChangeEvent.getNewValue() && WWindowPeer.log.isLoggable(PlatformLogger.Level.FINE)) {
                WWindowPeer.log.fine(" Assertion (newValue != true) failed for AppContext.GUI_DISPOSED ");
            }
            AppContext appContext = AppContext.getAppContext();
            synchronized (appContext) {
                appContext.remove(WWindowPeer.ACTIVE_WINDOWS_KEY);
                appContext.removePropertyChangeListener("guidisposed", this);
                KeyboardFocusManager.getCurrentKeyboardFocusManager().removePropertyChangeListener("activeWindow", WWindowPeer.activeWindowListener);
            }
        }
    }
    
    private static class ActiveWindowListener implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            Window target = (Window)propertyChangeEvent.getNewValue();
            if (target == null) {
                return;
            }
            AppContext targetToAppContext = SunToolkit.targetToAppContext(target);
            synchronized (targetToAppContext) {
                WWindowPeer wWindowPeer = (WWindowPeer)target.getPeer();
                List<WWindowPeer> list = (List)targetToAppContext.get(WWindowPeer.ACTIVE_WINDOWS_KEY);
                if (list != null) {
                    list.remove(wWindowPeer);
                    list.add(wWindowPeer);
                }
            }
            return;
        }
    }
}
